@@ -71,8 +71,16 @@ namespace Banshee.Sources
         }
     }
 
+    public delegate bool TrackEqualHandler (DatabaseTrackInfo a, TrackInfo b);
+
     public abstract class PrimarySource : DatabaseSource, IDisposable
     {
+        private TrackEqualHandler track_equal_handler;
+        public TrackEqualHandler TrackEqualHandler {
+            get { return track_equal_handler; }
+            protected set { track_equal_handler = value; }
+        }
+    
         protected ErrorSource error_source;
         protected bool error_source_visible = false;
 
@@ -94,6 +102,11 @@ namespace Banshee.Sources
             DELETE FROM CoreTracks WHERE PrimarySourceId = ?
         ");
 
+        private SchemaEntry<bool> expanded_schema;
+        public SchemaEntry<bool> ExpandedSchema {
+            get { return expanded_schema; }
+        }
+
         private int dbid;
         public int DbId {
             get {
@@ -116,6 +129,16 @@ namespace Banshee.Sources
             }
         }
 
+        private bool supports_playlists = true;
+        public virtual bool SupportsPlaylists {
+            get { return supports_playlists; }
+            protected set { supports_playlists = value; }
+        }
+
+        public virtual bool PlaylistsReadOnly {
+            get { return false; }
+        }
+
         public ErrorSource ErrorSource {
             get {
                 if (error_source == null) {
@@ -127,6 +150,13 @@ namespace Banshee.Sources
             }
         }
 
+
+        private bool is_local = false;
+        public bool IsLocal {
+            get { return is_local; }
+            protected set { is_local = value; }
+        }
+
         public delegate void TrackEventHandler (Source sender, TrackEventArgs args);
 
         public event TrackEventHandler TracksAdded;
@@ -138,7 +168,7 @@ namespace Banshee.Sources
         {
             return (primary_sources.ContainsKey (id)) ? primary_sources[id] : null;
         }
-
+        
         public virtual SafeUri UriAndTypeToSafeUri (TrackUriType type, string uri_field)
         {
             if (type == TrackUriType.RelativePath && BaseDirectory != null)
@@ -169,6 +199,22 @@ namespace Banshee.Sources
         {
         }
 
+        // Translators: this is a noun, referring to the harddisk
+        private string storage_name = Catalog.GetString ("Drive");
+        public string StorageName {
+            get { return storage_name; }
+            protected set { storage_name = value; }
+        }
+
+        public override bool? AutoExpand {
+            get { return ExpandedSchema.Get (); }
+        }
+
+        public override bool Expanded {
+            get { return ExpandedSchema.Get (); }
+            set { ExpandedSchema.Set (value); }
+        }
+
         public virtual void Dispose ()
         {
             if (Application.ShuttingDown)
@@ -192,17 +238,17 @@ namespace Banshee.Sources
         private void PrimarySourceInitialize ()
         {
             // Scope the tracks to this primary source
-            track_model.AddCondition (String.Format ("CoreTracks.PrimarySourceID = {0}", DbId));
+            DatabaseTrackModel.AddCondition (String.Format ("CoreTracks.PrimarySourceID = {0}", DbId));
 
             primary_sources[DbId] = this;
             
             // Load our playlists and smart playlists
-            foreach (PlaylistSource pl in PlaylistSource.LoadAll (DbId)) {
+            foreach (PlaylistSource pl in PlaylistSource.LoadAll (this)) {
                 AddChildSource (pl);
             }
 
             int sp_count = 0;
-            foreach (SmartPlaylistSource pl in SmartPlaylistSource.LoadAll (DbId)) {
+            foreach (SmartPlaylistSource pl in SmartPlaylistSource.LoadAll (this)) {
                 AddChildSource (pl);
                 sp_count++;
             }
@@ -224,7 +270,12 @@ namespace Banshee.Sources
                 // if we didn't have any and the list of default ones is empty atm).
                 if (sp_count > 0)
                     HaveCreatedSmartPlaylists = true;
+
             }
+
+            expanded_schema = new SchemaEntry<bool> (
+                String.Format ("sources.{0}", ParentConfigurationId), "expanded", true, "Is source expanded", "Is source expanded"
+            );
         }
 
         private bool HaveCreatedSmartPlaylists {
@@ -240,6 +291,11 @@ namespace Banshee.Sources
             );
         }
 
+        public virtual void CopyTrackTo (DatabaseTrackInfo track, SafeUri uri, BatchUserJob job)
+        {
+            Log.WarningFormat ("CopyTrackTo not implemented for source {0}", this);
+        }
+
         internal void NotifyTracksAdded ()
         {
             OnTracksAdded ();
@@ -263,12 +319,14 @@ namespace Banshee.Sources
 
         protected void OnErrorSourceUpdated (object o, EventArgs args)
         {
-            if (error_source.Count > 0 && !error_source_visible) {
-                AddChildSource (error_source);
-                error_source_visible = true;
-            } else if (error_source.Count <= 0 && error_source_visible) {
-                RemoveChildSource (error_source);
-                error_source_visible = false;
+            lock (error_source) {
+                if (error_source.Count > 0 && !error_source_visible) {
+                    error_source_visible = true;
+                    AddChildSource (error_source);
+                } else if (error_source.Count <= 0 && error_source_visible) {
+                    error_source_visible = false;
+                    RemoveChildSource (error_source);
+                }
             }
         }
 
@@ -318,7 +376,13 @@ namespace Banshee.Sources
         protected override void OnTracksChanged (params QueryField [] fields)
         {
             ThreadAssist.SpawnFromMain (delegate {
-                Reload ();
+                if (NeedsReloadWhenFieldsChanged (fields)) {
+                    Reload ();
+                } else {
+                    InvalidateCaches ();
+                }
+
+                System.Threading.Thread.Sleep (150);
 
                 TrackEventHandler handler = TracksChanged;
                 if (handler != null) {
@@ -368,20 +432,28 @@ namespace Banshee.Sources
             DeleteSelectedTracks (source.TrackModel as DatabaseTrackListModel);
         }
 
+        public void DeleteAllTracks (AbstractPlaylistSource source)
+        {
+            if (source.PrimarySource != this) {
+                Log.WarningFormat ("Cannot delete all tracks from {0} via primary source {1}", source, this);
+                return;
+            }
+            
+            ThreadAssist.SpawnFromMain (delegate {
+                CachedList<DatabaseTrackInfo> list = CachedList<DatabaseTrackInfo>.CreateFromModel (source.DatabaseTrackModel);
+                DeleteTrackList (list);
+            });
+        }
+
         protected override void DeleteSelectedTracks (DatabaseTrackListModel model)
         {
+            if (model == null) {
+                return;
+            }
+            
             ThreadAssist.SpawnFromMain (delegate {
-                if (model == null)
-                    return;
-
                 CachedList<DatabaseTrackInfo> list = CachedList<DatabaseTrackInfo>.CreateFromModelSelection (model);
                 DeleteTrackList (list);
-
-                OnTracksDeleted ();
-                OnUserNotifyUpdated ();
-                ThreadAssist.ProxyToMain (delegate {
-                    OnUpdated ();
-                });
             });
         }
 
@@ -420,6 +492,12 @@ namespace Banshee.Sources
 
             // Remove from database
             ServiceManager.DbConnection.Execute (remove_list_command, DateTime.Now, list.CacheId, list.CacheId);
+
+            ThreadAssist.ProxyToMain (delegate {
+                OnTracksDeleted ();
+                OnUserNotifyUpdated ();
+                OnUpdated ();
+            });
         }
 
         protected virtual void DeleteTrack (DatabaseTrackInfo track)
@@ -429,7 +507,9 @@ namespace Banshee.Sources
 
         public override bool AcceptsInputFromSource (Source source)
         {
-            return base.AcceptsInputFromSource (source) && source.Parent != this;
+            return base.AcceptsInputFromSource (source) && source.Parent != this
+                && (source.Parent is PrimarySource || source is PrimarySource)
+                && !(source.Parent is Banshee.Library.LibrarySource);
         }
 
         public override bool AddSelectedTracks (Source source)
@@ -441,9 +521,26 @@ namespace Banshee.Sources
 
             // Store a snapshot of the current selection
             CachedList<DatabaseTrackInfo> cached_list = CachedList<DatabaseTrackInfo>.CreateFromModelSelection (model);
-
-            System.Threading.ThreadPool.QueueUserWorkItem (AddTrackList, cached_list);
-
+            if (ThreadAssist.InMainThread) {
+                System.Threading.ThreadPool.QueueUserWorkItem (AddTrackList, cached_list);
+            } else {
+                AddTrackList (cached_list);
+            }
+            return true;
+        }
+        
+        public override bool AddAllTracks (Source source)
+        {
+            if (!AcceptsInputFromSource (source) || source.Count == 0)
+                return false;
+            
+            DatabaseTrackListModel model = (source as ITrackModelSource).TrackModel as DatabaseTrackListModel;
+            CachedList<DatabaseTrackInfo> cached_list = CachedList<DatabaseTrackInfo>.CreateFromModel (model);
+            if (ThreadAssist.InMainThread) {
+                System.Threading.ThreadPool.QueueUserWorkItem (AddTrackList, cached_list);
+            } else {
+                AddTrackList (cached_list);
+            }
             return true;
         }
 
@@ -471,6 +568,12 @@ namespace Banshee.Sources
             AddTrackJob.Total += (int) list.Count;
 
             foreach (DatabaseTrackInfo track in list) {
+                if (AddTrackJob.IsCancelRequested) {
+                    AddTrackJob.Finish ();
+                    IncrementAddedTracks ();
+                    break;
+                }
+                
                 if (track == null) {
                     IncrementAddedTracks ();
                     continue;
@@ -505,11 +608,14 @@ namespace Banshee.Sources
                 }
             }
 
-            if (notify || finished)
-                OnTracksAdded ();
-
-            if (finished)
-                OnUserNotifyUpdated ();
+            if (notify || finished) {
+                Banshee.Base.ThreadAssist.ProxyToMain (delegate {
+                    OnTracksAdded ();
+    
+                if (finished)
+                    OnUserNotifyUpdated ();
+                });
+            }
         }
 
         private bool delay_add_job = true;
@@ -533,6 +639,7 @@ namespace Banshee.Sources
                             "Adding {0} of {1} to {2}"), "{0}", "{1}", Name), 
                             Properties.GetStringList ("Icon.Name"));
                         add_track_job.DelayShow = DelayAddJob;
+                        add_track_job.CanCancel = true;
                         add_track_job.Register ();
                     }
                 }
@@ -560,6 +667,8 @@ namespace Banshee.Sources
         {
             ServiceManager.DbConnection.Execute (prune_artists_albums_command);
             base.PruneArtistsAlbums ();
+            DatabaseAlbumInfo.Reset ();
+            DatabaseArtistInfo.Reset ();
         }
     }
 }
