@@ -62,12 +62,17 @@ namespace Banshee.Sources
         public event SourceEventHandler ChildSourceRemoved;
 
         public delegate void OpenPropertiesDelegate ();
-        
-        protected Source (string generic_name, string name, int order)
+
+        protected Source (string generic_name, string name, int order) : this (generic_name, name, order, null)
+        {
+        }
+
+        protected Source (string generic_name, string name, int order, string type_unique_id)
         {
             GenericName = generic_name;
             Name = name;
             Order = order;
+            TypeUniqueId = type_unique_id;
 
             SourceInitialize ();
         }
@@ -102,7 +107,7 @@ namespace Banshee.Sources
         
         protected void OnSetupComplete ()
         {
-            ITrackModelSource tm_source = this as ITrackModelSource;
+            /*ITrackModelSource tm_source = this as ITrackModelSource;
             if (tm_source != null) {
                 tm_source.TrackModel.Parent = this;
                 ServiceManager.DBusServiceManager.RegisterObject (tm_source.TrackModel);
@@ -115,7 +120,7 @@ namespace Banshee.Sources
                         ServiceManager.DBusServiceManager.RegisterObject (exportable);
                     }
                 }
-            }
+            }*/
         }
 
         protected void Remove ()
@@ -151,7 +156,7 @@ namespace Banshee.Sources
         
         public virtual void MergeSourceInput (Source source, SourceMergeType mergeType)
         {
-            throw new NotImplementedException ();
+            Log.ErrorFormat ("MergeSourceInput not implemented by {0}", this);
         }
         
         public virtual SourceMergeType SupportedMergeTypes {
@@ -184,7 +189,9 @@ namespace Banshee.Sources
                 child_sources.Remove (child);
                 
                 if (ServiceManager.SourceManager.ActiveSource == child) {
-                    ServiceManager.SourceManager.SetActiveSource (this);
+                    if (CanActivate) {
+                        ServiceManager.SourceManager.SetActiveSource (this);
+                    }
                 }
                 
                 OnChildSourceRemoved (child);
@@ -231,6 +238,15 @@ namespace Banshee.Sources
             }
         }
         
+        public T GetInheritedProperty<T> (string name)
+        {
+            return Properties.Contains (name)
+                ? Properties.Get<T> (name)
+                : Parent != null 
+                    ? Parent.GetInheritedProperty<T> (name)
+                    : default (T);
+        }
+        
 #endregion
         
 #region Protected Methods
@@ -376,22 +392,26 @@ namespace Banshee.Sources
     
         protected virtual void OnChildSourceAdded (Source source)
         {
-            SourceEventHandler handler = ChildSourceAdded;
-            if (handler != null) {
-                SourceEventArgs args = new SourceEventArgs ();
-                args.Source = source;
-                handler (args);
-            }
+            ThreadAssist.ProxyToMain (delegate {
+                SourceEventHandler handler = ChildSourceAdded;
+                if (handler != null) {
+                    SourceEventArgs args = new SourceEventArgs ();
+                    args.Source = source;
+                    handler (args);
+                }
+            });
         }
         
         protected virtual void OnChildSourceRemoved (Source source)
         {
-            SourceEventHandler handler = ChildSourceRemoved;
-            if (handler != null) {
-                SourceEventArgs args = new SourceEventArgs ();
-                args.Source = source;
-                handler (args);
-            }
+            ThreadAssist.ProxyToMain (delegate {
+                SourceEventHandler handler = ChildSourceRemoved;
+                if (handler != null) {
+                    SourceEventArgs args = new SourceEventArgs ();
+                    args.Source = source;
+                    handler (args);
+                }
+            });
         }
         
         protected virtual void OnUpdated ()
@@ -444,10 +464,19 @@ namespace Banshee.Sources
         
         private string unique_id;
         public string UniqueId {
-            get { return unique_id ?? unique_id = String.Format ("{0}-{1}", this.GetType ().Name, TypeUniqueId); }
+            get {
+                if (unique_id == null && type_unique_id == null) {
+                    Log.ErrorFormat ("Creating Source.UniqueId for {0}, but TypeUniqueId is null; trace is {1}", this.Name, System.Environment.StackTrace);
+                }
+                return unique_id ?? unique_id = String.Format ("{0}-{1}", this.GetType ().Name, TypeUniqueId);
+            }
         }
         
-        protected abstract string TypeUniqueId { get; }
+        private string type_unique_id;
+        protected string TypeUniqueId {
+            get { return type_unique_id; }
+            set { type_unique_id = value; }
+        }
 
         public virtual bool CanRename {
             get { return false; }
@@ -456,14 +485,22 @@ namespace Banshee.Sources
         public virtual bool HasProperties {
             get { return false; }
         }
+        
+        public virtual bool HasViewableTrackProperties {
+            get { return false; }
+        }
+        
+        public virtual bool HasEditableTrackProperties {
+            get { return false; }
+        }
 
         public virtual string Name {
-            get { return properties.GetString ("Name"); }
+            get { return properties.Get<string> ("Name"); }
             set { properties.SetString ("Name", value); }
         }
 
         public virtual string GenericName {
-            get { return properties.GetString ("GenericName"); }
+            get { return properties.Get<string> ("GenericName"); }
             set { properties.SetString ("GenericName", value); }
         }
         
@@ -485,7 +522,7 @@ namespace Banshee.Sources
         }
                 
         public virtual string FilterQuery {
-            get { return properties.GetString ("FilterQuery"); }
+            get { return properties.Get<string> ("FilterQuery"); }
             set { properties.SetString ("FilterQuery", value); }
         }
         
@@ -514,9 +551,20 @@ namespace Banshee.Sources
         public virtual int Count {
             get { return 0; }
         }
-        
+
+        private string parent_conf_id;
+        public string ParentConfigurationId {
+            get {
+                if (parent_conf_id == null) {
+                    parent_conf_id = (Parent ?? this).UniqueId.Replace ('.', '_');
+                }
+                return parent_conf_id;
+            }
+        }
+
+        private string conf_id;
         public string ConfigurationId {
-            get { return (Parent == null ? UniqueId : Parent.UniqueId).Replace ('.', '_'); }
+            get { return conf_id ?? conf_id = UniqueId.Replace ('.', '_'); }
         }
 
         public virtual int FilteredCount { get { return Count; } }
@@ -538,9 +586,24 @@ namespace Banshee.Sources
             get { return duration_status_formatters.Count; }
         }
         
-        protected virtual int CurrentStatusFormat {
-            get { return ConfigurationClient.Get<int> (String.Format ("sources.{0}", ConfigurationId), "status_format", 0); }
-            set { ConfigurationClient.Set<int> (String.Format ("sources.{0}", ConfigurationId), "status_format", value); }
+        public virtual int CurrentStatusFormat {
+            get { return ConfigurationClient.Get<int> (String.Format ("sources.{0}", ParentConfigurationId), "status_format", 0); }
+            set { ConfigurationClient.Set<int> (String.Format ("sources.{0}", ParentConfigurationId), "status_format", value); }
+        }
+        
+        public SchemaEntry<T> CreateSchema<T> (string name)
+        {
+            return CreateSchema<T> (name, default(T), null, null);
+        }
+        
+        public SchemaEntry<T> CreateSchema<T> (string name, T defaultValue, string shortDescription, string longDescription)
+        {
+            return new SchemaEntry<T> (String.Format ("sources.{0}", ParentConfigurationId), name, defaultValue, shortDescription, longDescription); 
+        }
+        
+        public SchemaEntry<T> CreateSchema<T> (string ns, string name, T defaultValue, string shortDescription, string longDescription)
+        {
+            return new SchemaEntry<T> (String.Format ("sources.{0}.{1}", ParentConfigurationId, ns), name, defaultValue, shortDescription, longDescription); 
         }
         
         public void CycleStatusFormat ()
@@ -588,16 +651,17 @@ namespace Banshee.Sources
             return Name;
         }
         
-        string IService.ServiceName {
+        /*string IService.ServiceName {
             get { return String.Format ("{0}{1}", DBusServiceManager.MakeDBusSafeString (Name), "Source"); }
-        }
+        }*/
         
-        IDBusExportable IDBusExportable.Parent {
+        // FIXME: Replace ISource with IDBusExportable when it's enabled again
+        ISource ISource.Parent {
             get {
                 if (Parent != null) {
                     return ((Source)this).Parent;
                 } else {
-                    return ServiceManager.SourceManager;
+                    return null /*ServiceManager.SourceManager*/;
                 }
             }
         }
