@@ -32,6 +32,7 @@ using Gtk;
 
 using Banshee.Base;
 using Banshee.ServiceStack;
+using Hyena.Gui;
 
 namespace Banshee.Gui.Widgets
 {
@@ -51,6 +52,7 @@ namespace Banshee.Gui.Widgets
         private Button cancel_button;
         private uint update_delay_id;
         private uint progress_bounce_id;
+        private object tooltips;
         
         Banshee.Widgets.HigMessageDialog cancel_dialog;
         
@@ -65,11 +67,13 @@ namespace Banshee.Gui.Widgets
         
         private void BuildWidget ()
         {
+            ThreadAssist.AssertInMainThread ();
             ColumnSpacing = 5;
             RowSpacing = 2;
             
             icon = new Image ();
             
+            tooltips = TooltipSetter.CreateHost ();
             title_label = new Label ();
             title_label.Xalign = 0.0f;
             title_label.Ellipsize = Pango.EllipsizeMode.End;
@@ -155,21 +159,28 @@ namespace Banshee.Gui.Widgets
             cancel_dialog = null;
         }
         
+        private void SetTitle (string new_title)
+        {
+            if (String.IsNullOrEmpty (new_title)) {
+                title_label.Hide ();
+            } else {
+                title_label.Markup = String.Format ("<small><b>{0}</b></small>", GLib.Markup.EscapeText (new_title));
+                title_label.Show ();
+            }
+            title = new_title;
+        }
+        
         private void UpdateFromJob ()
         {
+            ThreadAssist.AssertInMainThread ();
             if (cancel_dialog != null && !job.CanCancel) {
                 cancel_dialog.Respond (Gtk.ResponseType.Cancel);
             }
             
-            if (title != job.Title) {
-                if (String.IsNullOrEmpty (job.Title)) {
-                    title_label.Hide ();
-                } else {
-                    title_label.Markup = String.Format ("<small><b>{0}</b></small>", 
-                        GLib.Markup.EscapeText (job.Title));
-                    title_label.Show ();
-                }
-                title = job.Title;
+            if (job.IsCancelRequested) {
+                SetTitle (Catalog.GetString ("Stopping..."));
+            } else if (title != job.Title) {
+                SetTitle (job.Title);
             }
             
             if (status != job.Status) {
@@ -177,6 +188,7 @@ namespace Banshee.Gui.Widgets
                     status_label.Hide ();
                 } else {
                     status_label.Markup = String.Format ("<small>{0}</small>", GLib.Markup.EscapeText (job.Status));
+                    TooltipSetter.Set (tooltips, status_label, job.Status);
                     status_label.Show ();
                 }
                 status = job.Status;
@@ -186,14 +198,14 @@ namespace Banshee.Gui.Widgets
                 UpdateIcons ();
             } else {
                 for (int i = 0; i < job.IconNames.Length; i++) {
-                    if (!icon_names[i].Equals (job.IconNames[i])) {
+                    if (icon_names[i] != job.IconNames[i]) {
                         UpdateIcons ();
                         break;
                     }
                 }
             }
             
-            cancel_button.Sensitive = job.CanCancel;
+            cancel_button.Sensitive = job.CanCancel && !job.IsCancelRequested;
             
             if (job.Progress == 0 && progress_bounce_id > 0) {
                 return;
@@ -235,6 +247,15 @@ namespace Banshee.Gui.Widgets
             if (icon_pixbuf != null) {
                 icon.Pixbuf = icon_pixbuf;
                 icon.Show ();
+                return;
+            }
+
+            icon_pixbuf = new Gdk.Pixbuf (icon_names[0]);
+            if (icon_pixbuf != null) {
+                Gdk.Pixbuf scaled = icon_pixbuf.ScaleSimple (22, 22, Gdk.InterpType.Bilinear);
+                icon_pixbuf.Dispose ();
+                icon.Pixbuf = scaled;
+                icon.Show ();
             }
         }
         
