@@ -35,47 +35,72 @@ using Hyena.Gui.Theming;
 
 namespace Hyena.Data.Gui
 {
-    public class ColumnCellText : ColumnCell
+    public class ColumnCellText : ColumnCell, ISizeRequestCell, ITextCell
     {
+        internal const int Spacing = 4;
+
         public delegate string DataHandler ();
     
         private Pango.Weight font_weight = Pango.Weight.Normal;
         private Pango.EllipsizeMode ellipsize_mode = Pango.EllipsizeMode.End;
         private Pango.Alignment alignment = Pango.Alignment.Left;
+        private double opacity = 1.0;
         private int text_width;
         private int text_height;
+        private string text_format = null;
+        protected string MinString, MaxString;
         
         public ColumnCellText (string property, bool expand) : base (property, expand)
         {
         }
+
+        protected void SetMinMaxStrings (object min_max)
+        {
+            SetMinMaxStrings (min_max, min_max);
+        }
+        
+        protected void SetMinMaxStrings (object min, object max)
+        {
+            // Set the min/max strings from the min/max objects
+            MinString = GetText (min);
+            MaxString = GetText (max);
+            RestrictSize = true;
+        }
     
         public override void Render (CellContext context, StateType state, double cellWidth, double cellHeight)
         {
-            string text = Text;
+            string text = GetText (BoundObject);
             if (String.IsNullOrEmpty (text)) {
                 return;
             }
-        
+
+            // TODO why doesn't Spacing (eg 4 atm) work here instead of 8?  Rendering
+            // seems to be off when changed to Spacing/4
             context.Layout.Width = (int)((cellWidth - 8) * Pango.Scale.PangoScale);
             context.Layout.FontDescription.Weight = font_weight;
             context.Layout.Ellipsize = EllipsizeMode;
             context.Layout.Alignment = alignment;
+
+            if (text_format == null) {
+                context.Layout.SetText (text);
+            } else {
+                context.Layout.SetText (String.Format (text_format, text));
+            }
             
-            context.Layout.SetText (text);
             context.Layout.GetPixelSize (out text_width, out text_height);
             
-            context.Context.MoveTo (4, ((int)cellHeight - text_height) / 2);
+            context.Context.MoveTo (Spacing, ((int)cellHeight - text_height) / 2);
             Cairo.Color color = context.Theme.Colors.GetWidgetColor (
                 context.TextAsForeground ? GtkColorClass.Foreground : GtkColorClass.Text, state);
-            if (!context.Sensitive) {
-                color.A = 0.3;
-            }
+            color.A = (!context.Sensitive) ? 0.3 : opacity;
             context.Context.Color = color;
+
             PangoCairoHelper.ShowLayout (context.Context, context.Layout);
         }
         
-        protected virtual string Text {
-            get { return BoundObject == null ? String.Empty : BoundObject.ToString (); }
+        protected virtual string GetText (object obj)
+        {
+            return obj == null ? String.Empty : obj.ToString ();
         }
         
         protected int TextWidth {
@@ -85,6 +110,11 @@ namespace Hyena.Data.Gui
         protected int TextHeight {
             get { return text_height; }
         }
+
+        public string TextFormat {
+            get { return text_format; }
+            set { text_format = value; }
+        }
         
         protected Pango.Alignment Alignment {
             get { return alignment; }
@@ -101,6 +131,11 @@ namespace Hyena.Data.Gui
             set { ellipsize_mode = value; }
         }
         
+        public virtual double Opacity {
+            get { return opacity; }
+            set { opacity = value; }
+        }
+        
         internal static int ComputeRowHeight (Widget widget)
         {
             int w_width, row_height;
@@ -110,5 +145,36 @@ namespace Hyena.Data.Gui
             layout.Dispose ();
             return row_height + 8;
         }
+
+
+        #region ISizeRequestCell implementation 
+        
+        public void GetWidthRange (Pango.Layout layout, out int min, out int max)
+        {
+            int height;
+            min = max = -1;
+            
+            if (!String.IsNullOrEmpty (MinString)) {
+                layout.SetText (MinString);
+                layout.GetPixelSize (out min, out height);
+                min += 2*Spacing;
+                //Console.WriteLine ("for {0} got min {1} for {2}", this, min, MinString);
+            }
+
+            if (!String.IsNullOrEmpty (MaxString)) {
+                layout.SetText (MaxString);
+                layout.GetPixelSize (out max, out height);
+                max += 2*Spacing;
+                //Console.WriteLine ("for {0} got max {1} for {2}", this, max, MaxString);
+            }
+        }
+        
+        private bool restrict_size = false;
+        public bool RestrictSize {
+            get { return restrict_size; }
+            set { restrict_size = value; }
+        }
+        
+        #endregion
     }
 }
