@@ -188,6 +188,10 @@ namespace Hyena.Data.Gui
                 case Gdk.Key.KP_Enter:
                     handled = ActivateSelection ();
                     break;
+
+                case Gdk.Key.Escape:
+                    handled = CancelColumnDrag ();
+                    break;
                 
                 case Gdk.Key.space:
                     if (Selection != null && Selection.FocusedIndex != 1) {
@@ -343,7 +347,7 @@ namespace Hyena.Data.Gui
             }
             
             Gtk.Drag.SourceUnset (this);
-            
+
             Column column = GetColumnForResizeHandle (x);
             if (column != null) {
                 resizing_column_index = GetCachedColumnForColumn (column).Index;
@@ -379,43 +383,49 @@ namespace Hyena.Data.Gui
             
             ProxyEventToCell (evnt, true);
             
-            if (Selection != null && evnt.Button == 1 && evnt.Type != Gdk.EventType.TwoButtonPress && 
-                (evnt.State & Gdk.ModifierType.ControlMask) == 0 && Selection.Contains (row_index)) {
-                return true;
-            }
-
             object item = model[row_index];
             if (item == null) {
                 return true;
             }
 
             if (evnt.Button == 1 && evnt.Type == Gdk.EventType.TwoButtonPress) {
+                // Double clicked
                 OnRowActivated ();
             } else if (Selection != null) {
                 if ((evnt.State & Gdk.ModifierType.ControlMask) != 0) {
                     if (evnt.Button == 3) {
+                        // Right clicked with ctrl pressed, so make sure row selected
                         if (!Selection.Contains (row_index)) {
                             Selection.Select (row_index);
                         }
                     } else {
+                        // Normal ctrl-click, so toggle
                         Selection.ToggleSelect (row_index);
                     }
                 } else if ((evnt.State & Gdk.ModifierType.ShiftMask) != 0) {
+                    // Shift-click, so select from first-row-selected (if any) to the current row
                     Selection.SelectFromFirst (row_index, true);
                 } else {
                     if (evnt.Button == 3) {
+                        // Normal right-click, make sure row is only row selected
                         if (!Selection.Contains (row_index)) {
                             Selection.Clear (false);
                             Selection.Select (row_index);
                         }
                     } else {
-                        Selection.Clear (false);
-                        Selection.Select (row_index);
+                        // Normal click, if row not already selected, select only it right now,
+                        // but if it's already selected, wait until the Release to unselect any others so that
+                        // drag and drop of 2+ items works.
+                        if (!Selection.Contains (row_index)) {
+                            Selection.Clear (false);
+                            Selection.Select (row_index);
+                        }
                     }
                 }
 
                 FocusRow (row_index);
 
+                // Now that we've worked out the selections, open the context menu
                 if (evnt.Button == 3) {
                     OnPopupMenu ();
                 }
@@ -440,12 +450,10 @@ namespace Hyena.Data.Gui
                 GdkWindow.Cursor = null;
                 return true;
             }
-            
-            if (pressed_column_index >= 0 && pressed_column_is_dragging) {
-                pressed_column_is_dragging = false;
-                pressed_column_index = -1;
-                GdkWindow.Cursor = null;
-                QueueDraw ();
+
+            if (pressed_column_drag_started) {
+                CancelColumnDrag ();
+                pressed_column_drag_started = false;
                 return true;
             }
             
@@ -459,13 +467,31 @@ namespace Hyena.Data.Gui
 
             return true;
         }
-        
+
         private bool OnHeaderButtonRelease (Gdk.EventButton evnt)
         {
             if (pressed_column_index >= 0 && pressed_column_index < column_cache.Length) {
                 Column column = column_cache[pressed_column_index].Column;
                 if (column != null && Model is ISortable && column is ISortableColumn) {
-                    ((ISortable)Model).Sort ((ISortableColumn)column);
+                    ISortableColumn sort_column = column as ISortableColumn;
+                    ISortable sortable = Model as ISortable;
+
+                    // Change the sort-type with every click
+                    switch (sort_column.SortType) {
+                        case SortType.Ascending:    sort_column.SortType = SortType.Descending; break;
+                        case SortType.Descending:   sort_column.SortType = SortType.None; break;
+                        case SortType.None:         sort_column.SortType = SortType.Ascending; break;
+                    }
+
+                    // If we're switching from a different column, or we aren't reorderable, make sure sort type isn't None
+                    if ((sort_column != ColumnController.SortColumn || !IsEverReorderable) && sort_column.SortType == SortType.None) {
+                        sort_column.SortType = SortType.Ascending;
+                    }
+
+                    sortable.Sort (sort_column);
+                    ColumnController.SortColumn = sort_column;
+                    IsReorderable = sortable.SortColumn == null || sortable.SortColumn.SortType == SortType.None;
+
                     Model.Reload ();
                     RecalculateColumnSizes ();
                     RegenerateColumnCache ();
@@ -500,10 +526,14 @@ namespace Hyena.Data.Gui
                 return true;
             }
             
-            if (Selection != null && Selection.Contains (row_index) && Selection.Count > 1) {
-                Selection.Clear (false);
-                Selection.Select (row_index);
-                FocusRow (row_index);
+            //if (Selection != null && Selection.Contains (row_index) && Selection.Count > 1) {
+            if (Selection != null && evnt.Button == 1 && Hyena.Gui.GtkUtilities.NoImportantModifiersAreSet ()) {
+                if (Selection.Count > 1) {
+                    Selection.Clear (false);
+                    Selection.Select (row_index);
+                    FocusRow (row_index);
+                    InvalidateList ();    
+                }
             }
             
             return true;
@@ -518,6 +548,7 @@ namespace Hyena.Data.Gui
             if (pressed_column_index >= 0 && !pressed_column_is_dragging && 
                 Gtk.Drag.CheckThreshold (this, pressed_column_x_start, 0, x, 0)) {
                 pressed_column_is_dragging = true;
+                pressed_column_drag_started = true;
                 InvalidateHeader ();
                 InvalidateList ();
             }
@@ -527,9 +558,9 @@ namespace Hyena.Data.Gui
             if (OnMotionNotifyEvent (x)) {
                 return true;
             }
-            
-            GdkWindow.Cursor = (resizing_column_index >= 0 || GetColumnForResizeHandle (x) != null) &&
-                header_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y) 
+
+            GdkWindow.Cursor = header_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y) && 
+                (resizing_column_index >= 0 || GetColumnForResizeHandle (x) != null)
                 ? resize_x_cursor 
                 : null;
             
@@ -562,9 +593,11 @@ namespace Hyena.Data.Gui
                     // Moving from right to left
                     reorder = pressed_column_x_drag <= swap_column_c.X1 + swap_column_c.Width / 2;
                 } else if (swap_column_c.Index > pressed_column_index) {
-                    // Moving from left to right
-                    reorder = pressed_column_x_drag + column_cache[pressed_column_index].Width >= 
-                        swap_column_c.X1 + swap_column_c.Width / 2;
+                    if (column_cache.Length > pressed_column_index && pressed_column_index >= 0) {
+                        // Moving from left to right
+                        reorder = pressed_column_x_drag + column_cache[pressed_column_index].Width >= 
+                            swap_column_c.X1 + swap_column_c.Width / 2;
+                    }
                 }
                 
                 if (reorder) {
@@ -616,6 +649,18 @@ namespace Hyena.Data.Gui
                 }
             }
         }
+
+        private bool CancelColumnDrag ()
+        {
+            if (pressed_column_index >= 0 && pressed_column_is_dragging) {
+                pressed_column_is_dragging = false;
+                pressed_column_index = -1;
+                GdkWindow.Cursor = null;
+                QueueDraw ();
+                return true;
+            }
+            return false;
+        }
         
         protected int GetRowAtY (int y)
         {
