@@ -40,7 +40,7 @@ namespace Hyena.Data.Gui
 {
     public delegate int ListViewRowHeightHandler (Widget widget);
 
-    public partial class ListView<T> : Widget
+    public partial class ListView<T> : ListViewBase
     {
         private Cairo.Context cairo_context;
         private CellContext cell_context;
@@ -58,9 +58,14 @@ namespace Hyena.Data.Gui
             base.OnStyleSet (old_style);
             RecomputeRowHeight = true;
             theme = new GtkTheme (this);
+
+            // Save the drawable so we can reuse it
+            Gdk.Drawable drawable = cell_context != null ? cell_context.Drawable : null;
+
             cell_context = new CellContext ();
             cell_context.Theme = theme;
             cell_context.Widget = this;
+            cell_context.Drawable = drawable;
         }
          
         protected override bool OnExposeEvent (EventExpose evnt)
@@ -79,8 +84,12 @@ namespace Hyena.Data.Gui
             if (header_visible && column_controller != null) {
                 PaintHeader (damage);
             }
+           
             Theme.DrawFrameBorder (cairo_context, Allocation);
-            PaintRows(damage);
+            if (Model != null) {
+                PaintRows(damage);
+            }
+            
             PaintDraggingColumn (damage);
             
             ((IDisposable)cairo_context.Target).Dispose ();
@@ -107,26 +116,28 @@ namespace Hyena.Data.Gui
             cell_context.Sensitive = true;
             cell_context.TextAsForeground = true;
 
+            bool have_drawn_separator = false;
+
             for (int ci = 0; ci < column_cache.Length; ci++) {
                 if (pressed_column_is_dragging && pressed_column_index == ci) {
                     continue;
                 }
                 
-                cell_area.X = column_cache[ci].X1 + Theme.TotalBorderWidth + header_rendering_alloc.X - (int)hadjustment.Value;
+                cell_area.X = column_cache[ci].X1 + Theme.TotalBorderWidth + header_rendering_alloc.X - HadjustmentValue;
                 cell_area.Width = column_cache[ci].Width;
-                PaintHeaderCell (cell_area, ci, false);
+                PaintHeaderCell (cell_area, ci, false, ref have_drawn_separator);
             }
             
             if (pressed_column_is_dragging && pressed_column_index >= 0) {
-                cell_area.X = pressed_column_x_drag + Allocation.X - (int)hadjustment.Value;
+                cell_area.X = pressed_column_x_drag + Allocation.X - HadjustmentValue;
                 cell_area.Width = column_cache[pressed_column_index].Width;
-                PaintHeaderCell (cell_area, pressed_column_index, true);
+                PaintHeaderCell (cell_area, pressed_column_index, true, ref have_drawn_separator);
             }
             
             cairo_context.ResetClip ();
         }
         
-        private void PaintHeaderCell (Rectangle area, int ci, bool dragging)
+        private void PaintHeaderCell (Rectangle area, int ci, bool dragging, ref bool have_drawn_separator)
         {
             if (ci < 0 || column_cache.Length <= ci)
                 return;
@@ -157,7 +168,9 @@ namespace Hyena.Data.Gui
                 cairo_context.Restore ();
             }
             
-            if (!dragging && ci < column_cache.Length - 1) {
+            if (!dragging && ci < column_cache.Length - 1 && (have_drawn_separator || 
+                column_cache[ci].MaxWidth != column_cache[ci].MinWidth)) {
+                have_drawn_separator = true;
                 Theme.DrawHeaderSeparator (cairo_context, area, area.Right);
             }
         }
@@ -169,7 +182,7 @@ namespace Hyena.Data.Gui
             if (sort_column_index != -1 && (!pressed_column_is_dragging || pressed_column_index != sort_column_index)) {
                 CachedColumn col = column_cache[sort_column_index];
                 Theme.DrawRowRule (cairo_context,
-                    list_rendering_alloc.X + col.X1 - (int)hadjustment.Value,
+                    list_rendering_alloc.X + col.X1 - HadjustmentValue,
                     header_rendering_alloc.Bottom + Theme.BorderWidth,
                     col.Width, list_rendering_alloc.Height + Theme.InnerBorderWidth * 2);
             }
@@ -181,7 +194,7 @@ namespace Hyena.Data.Gui
             cell_context.Clip = clip;
             cell_context.TextAsForeground = false;
             
-            int vadjustment_value = (int)vadjustment.Value;
+            int vadjustment_value = VadjustmentValue;
             int first_row = vadjustment_value / RowHeight;
             int last_row = Math.Min (model.Count, first_row + RowsInView);
             int offset = list_rendering_alloc.Y - vadjustment_value % RowHeight;
@@ -189,7 +202,7 @@ namespace Hyena.Data.Gui
             Rectangle selected_focus_alloc = Rectangle.Zero;
             Rectangle single_list_alloc = new Rectangle ();
             
-            single_list_alloc.X = list_rendering_alloc.X - (int)(hadjustment.Value);
+            single_list_alloc.X = list_rendering_alloc.X - HadjustmentValue;
             single_list_alloc.Y = offset;
             single_list_alloc.Width = list_rendering_alloc.Width;
             single_list_alloc.Height = RowHeight;
@@ -216,16 +229,7 @@ namespace Hyena.Data.Gui
                             single_list_alloc.Width, single_list_alloc.Height);
                     }
                     
-                    if (ri == drag_reorder_row_index && Reorderable) {
-                        cairo_context.Save ();
-                        cairo_context.LineWidth = 1.0;
-                        cairo_context.Antialias = Cairo.Antialias.None;
-                        cairo_context.MoveTo (single_list_alloc.Left, single_list_alloc.Top);
-                        cairo_context.LineTo (single_list_alloc.Right, single_list_alloc.Top);
-                        cairo_context.Color = Theme.Colors.GetWidgetColor (GtkColorClass.Text, StateType.Normal);
-                        cairo_context.Stroke ();
-                        cairo_context.Restore ();
-                    }
+                    PaintReorderLine (ri, single_list_alloc);
                     
                     if (Selection != null && Selection.FocusedIndex == ri && !Selection.Contains (ri) && HasFocus) {
                         CairoCorners corners = CairoCorners.All;
@@ -254,6 +258,9 @@ namespace Hyena.Data.Gui
                 single_list_alloc.Y += single_list_alloc.Height;
             }
             
+            // In case the user is dragging to the end of the list
+            PaintReorderLine (last_row, single_list_alloc);
+            
             if (selection_height > 0) {
                 Theme.DrawRowSelection (cairo_context, list_rendering_alloc.X, selection_y, 
                     list_rendering_alloc.Width, selection_height);
@@ -273,6 +280,20 @@ namespace Hyena.Data.Gui
             
             cairo_context.ResetClip ();
         }
+        
+        private void PaintReorderLine (int row_index, Rectangle single_list_alloc)
+        {
+            if (row_index == drag_reorder_row_index && IsReorderable) {
+                cairo_context.Save ();
+                cairo_context.LineWidth = 1.0;
+                cairo_context.Antialias = Cairo.Antialias.None;
+                cairo_context.MoveTo (single_list_alloc.Left, single_list_alloc.Top);
+                cairo_context.LineTo (single_list_alloc.Right, single_list_alloc.Top);
+                cairo_context.Color = Theme.Colors.GetWidgetColor (GtkColorClass.Text, StateType.Normal);
+                cairo_context.Stroke ();
+                cairo_context.Restore ();
+            }
+        }
 
         private void PaintRow (int row_index, Rectangle area, StateType state)
         {
@@ -282,6 +303,7 @@ namespace Hyena.Data.Gui
             
             object item = model[row_index];
             bool sensitive = IsRowSensitive (item);
+            bool bold = IsRowBold (item);
             
             Rectangle cell_area = new Rectangle ();
             cell_area.Height = RowHeight;
@@ -294,23 +316,28 @@ namespace Hyena.Data.Gui
                 
                 cell_area.Width = column_cache[ci].Width;
                 cell_area.X = column_cache[ci].X1 + area.X;
-                PaintCell (item, ci, row_index, cell_area, sensitive, state, false);
+                PaintCell (item, ci, row_index, cell_area, sensitive, bold, state, false);
             }
             
             if (pressed_column_is_dragging && pressed_column_index >= 0) {
                 cell_area.Width = column_cache[pressed_column_index].Width;
                 cell_area.X = pressed_column_x_drag + list_rendering_alloc.X - list_interaction_alloc.X;
-                PaintCell (item, pressed_column_index, row_index, cell_area, sensitive, state, true);
+                PaintCell (item, pressed_column_index, row_index, cell_area, sensitive, bold, state, true);
             }
         }
         
-        private void PaintCell (object item, int column_index, int row_index, Rectangle area, bool sensitive,
+        private void PaintCell (object item, int column_index, int row_index, Rectangle area, bool sensitive, bool bold,
             StateType state, bool dragging)
         {
             ColumnCell cell = column_cache[column_index].Column.GetCell (0);
             cell.BindListItem (item);
             ColumnCellDataProvider (cell, item);
             
+            ITextCell text_cell = cell as ITextCell;
+            if (text_cell != null) {
+                text_cell.FontWeight = bold ? Pango.Weight.Bold : Pango.Weight.Normal;
+            }
+            
             if (dragging) {
                 Cairo.Color fill_color = Theme.Colors.GetWidgetColor (GtkColorClass.Base, StateType.Normal);
                 fill_color.A = 0.5;
@@ -335,7 +362,7 @@ namespace Hyena.Data.Gui
             
             CachedColumn column = column_cache[pressed_column_index];
             
-            int x = pressed_column_x_drag + Allocation.X + 1 - (int)hadjustment.Value;
+            int x = pressed_column_x_drag + Allocation.X + 1 - HadjustmentValue;
             
             Cairo.Color fill_color = Theme.Colors.GetWidgetColor (GtkColorClass.Base, StateType.Normal);
             fill_color.A = 0.45;
@@ -409,7 +436,7 @@ namespace Hyena.Data.Gui
         }
         
         private int row_height = 32;
-        private int RowHeight {
+        protected int RowHeight {
             get {
                 if (RecomputeRowHeight) {
                     row_height = RowHeightProvider != null 
