@@ -44,8 +44,8 @@ namespace Hyena.Query
         //public static readonly Operator NotEqual           = new Operator ("notEqual", "!= {0}", true, "!=", "!:");
         //public static readonly Operator LessThanEqual      = new Operator ("lessThanEquals", "<= {0}", "<=");
         //public static readonly Operator GreaterThanEqual   = new Operator ("greaterThanEquals", ">= {0}", ">=");
-        public static readonly Operator LessThan           = new Operator ("lessThan", "< {0}", "<"); // Before
-        public static readonly Operator GreaterThan        = new Operator ("greaterThan", "> {0}", ">"); // After
+        public static readonly Operator LessThan           = new Operator ("lessThan", Catalog.GetString ("before"), "< {0}", true, "<");
+        public static readonly Operator GreaterThan        = new Operator ("greaterThan", Catalog.GetString ("after"), "> {0}", ">");
 
         protected DateTime value = DateTime.Now;
 
@@ -88,7 +88,7 @@ namespace Hyena.Query
             IsEmpty = false;
         }
 
-        public void LoadString (string val, bool isRelative)
+        public override void LoadString (string val)
         {
             try {
                 SetValue (DateTime.Parse (val));
@@ -100,7 +100,7 @@ namespace Hyena.Query
         public override void ParseXml (XmlElement node)
         {
             try {
-                LoadString (node.InnerText, node.HasAttribute ("type") && node.GetAttribute ("type") == "rel");
+                LoadString (node.InnerText);
             } catch {
                 IsEmpty = true;
             }
